//Created by: blue77//
//////////////////////
using System;
using Server.Network; 
using Server.Prompts; 
using Server.Items; 
using Server.Mobiles;
using Server.Targeting;
using System.Collections;
using Server.Factions;

namespace Server.Items 
{ 
   public class TamingTarget : Target 
   { 
      private PetTamingDeed m_Deed; 

      public TamingTarget( PetTamingDeed deed ) : base( 5, false, TargetFlags.None ) 
      { 
         m_Deed = deed; 
      } 
      
      public static bool CheckMastery( Mobile from, BaseCreature t )
	{
		BaseCreature familiar = (BaseCreature)Spells.Necromancy.SummonFamiliarSpell.Table[from];

		if ( familiar != null && !familiar.Deleted && familiar is DarkWolfFamiliar )
		{
			if ( t is DireWolf || t is GreyWolf || t is TimberWolf || t is WhiteWolf )
			return true;
		}

			return false;
	}

	

	public static void Scale( BaseCreature t, double scalar, bool scaleStats )
		{
		if ( scaleStats )
		{
		if ( t.RawStr > 0 )
		t.RawStr = (int)Math.Max( 1, t.RawStr * scalar );

		if ( t.RawDex > 0 )
		t.RawDex = (int)Math.Max( 1, t.RawDex * scalar );

		if ( t.HitsMaxSeed > 0 )
		{
		t.HitsMaxSeed = (int)Math.Max( 1, t.HitsMaxSeed * scalar );
		t.Hits = t.Hits;
		}

		if ( t.StamMaxSeed > 0 )
		{
		t.StamMaxSeed = (int)Math.Max( 1, t.StamMaxSeed * scalar );
		t.Stam = t.Stam;
		}
	}

		for ( int i = 0; i < t.Skills.Length; ++i )
		t.Skills[i].Base *= scalar;
	}
      

      protected override void OnTarget( Mobile from, object target ) 
      { 
         if ( target is BaseCreature ) 
         { 
            BaseCreature t = ( BaseCreature ) target; 

            if ( t.Controled == true )
            { 
               from.SendMessage( "That pet is already tame!" );
            } 
            else if ( t.Tamable == false ) 
            { 
               from.SendMessage( "That creature cannot be tamed!" ); 
            }
	    else if ( from.Female && !t.AllowFemaleTamer )
		{
		from.SendMessage( "That creature can only be tamed by males" ); // That creature can only be tamed by males.
		}
	   else if ( !from.Female && !t.AllowMaleTamer )
		{
		from.SendMessage( "That creature can only be tamed by females" ); // That creature can only be tamed by females.
		}
	   else if ( from.Followers + t.ControlSlots > from.FollowersMax )
		{
		from.SendMessage( "You have too many followers to tame that creature" ); // You have too many followers to tame that creature.
		}
	   else if ( t.Owners.Count >= BaseCreature.MaxOwners && !t.Owners.Contains( from ) )
		{
		from.SendMessage( "This animal has had too many owners and is too upset for you to tame" ); // This animal has had too many owners and is too upset for you to tame.
		}
else if ( CheckMastery( from, t ) || from.Skills[SkillName.AnimalTaming].Value <= t.MinTameSkill )
		{
		from.SendMessage( "You do not have enough skill to tame this creature");
		}		
	  else if ( CheckMastery( from, t ) || from.Skills[SkillName.AnimalTaming].Value >= t.MinTameSkill )
		{
		FactionWarHorse warHorse = t as FactionWarHorse;

			if ( warHorse != null )
			{
			Faction faction = Faction.Find( from );

			if ( faction == null || faction != warHorse.Faction )
			{
			from.SendMessage( "You cannot tame this creature" ); // You cannot tame this creature.
			return;
			}
		}
			
		            
            else  
             

		{
			if ( t.Owners.Count == 0 ) // First tame
			{
			if ( t.Paralyzed )
			Scale( t, 0.86, false ); // 86% of original skills if they were paralyzed during the taming
			else
			Scale( t, 0.90, false ); // 90% of original skills
			}
			if ( t.SubdueBeforeTame )
			{
			Scale( t, 0.50, true ); // Creatures which must be subdued take an additional 50% loss of skills and stats
			}

			if ( t.ControlMaster == from )
			{
			from.SendLocalizedMessage( 502797 ); // That wasn't even challenging.
			}
			else
			{
			t.PrivateOverheadMessage( MessageType.Regular, 0x3B2, 502799, from.NetState ); // It seems to accept you as master.
			t.Owners.Add( from );
			t.SetControlMaster( from );
			t.IsBonded = false;
			m_Deed.Delete(); // Delete the deed
			return;
			}               
		} 
	} 
} 
}
}


   public class PetTamingDeed : Item // Create the item class which is derived from the base item class 
   { 
      [Constructable] 
      public PetTamingDeed() : base( 0x14F0 ) 
      { 
         Weight = 1.0; 
         Name = "a pet taming deed"; 
         LootType = LootType.Blessed; 
	 Hue = 1160;
      } 

      public PetTamingDeed( Serial serial ) : base( serial ) 
      { 
      } 

      public override void Serialize( GenericWriter writer ) 
      { 
         base.Serialize( writer ); 

         writer.Write( (int) 0 ); // version 
      } 

      public override void Deserialize( GenericReader reader ) 
      { 
         base.Deserialize( reader ); 
         LootType = LootType.Blessed; 

         int version = reader.ReadInt(); 
      } 

      public override bool DisplayLootType{ get{ return false; } } 

      public override void OnDoubleClick( Mobile from ) // Override double click of the deed to call our target 
      { 
         if ( !IsChildOf( from.Backpack ) ) // Make sure its in their pack 
         { 
             from.SendLocalizedMessage( 1042001 ); // That must be in your pack for you to use it. 
         } 
         else 
         { 
            from.SendMessage( "Choose the animal you wish to tame." );  
            from.Target = new TamingTarget( this ); // Call our target 
          } 
      }    
   } 
}
